@@ -13,23 +13,34 @@ import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.RemoveResourceS
 import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.ResourceTD;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.ResourceTDDescriptor;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.ResourceTDType;
+import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.SaveResourceSession;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.StringResourceTD;
 import org.gcube.portlets.user.td.gwtservice.shared.tr.resources.TableResourceTD;
 import org.gcube.portlets.user.td.gwtservice.shared.uriresolver.UriResolverSession;
 import org.gcube.portlets.user.td.gwtservice.shared.user.UserInfo;
+import org.gcube.portlets.user.td.resourceswidget.client.charts.ChartViewerDialog;
+import org.gcube.portlets.user.td.resourceswidget.client.custom.ResourceTDTypeButtonCell;
 import org.gcube.portlets.user.td.resourceswidget.client.properties.ResourceTDDescriptorProperties;
 import org.gcube.portlets.user.td.resourceswidget.client.resources.ResourceBundle;
+import org.gcube.portlets.user.td.resourceswidget.client.save.SaveResourceWizard;
 import org.gcube.portlets.user.td.resourceswidget.client.utils.UtilsGXT3;
 import org.gcube.portlets.user.td.widgetcommonevent.client.event.SessionExpiredEvent;
 import org.gcube.portlets.user.td.widgetcommonevent.client.type.SessionExpiredType;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.TRId;
 import org.gcube.portlets.user.td.widgetcommonevent.shared.uriresolver.ApplicationType;
+import org.gcube.portlets.user.td.wizardwidget.client.WizardListener;
 
 import com.allen_sauer.gwt.log.client.Log;
 import com.google.gwt.cell.client.AbstractCell;
+import com.google.gwt.cell.client.Cell.Context;
 import com.google.gwt.core.client.GWT;
+import com.google.gwt.core.client.RunAsyncCallback;
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.core.client.Scheduler.ScheduledCommand;
+import com.google.gwt.dom.client.BrowserEvents;
+import com.google.gwt.dom.client.Document;
+import com.google.gwt.dom.client.Element;
+import com.google.gwt.dom.client.NativeEvent;
 import com.google.gwt.event.logical.shared.SelectionEvent;
 import com.google.gwt.event.logical.shared.SelectionHandler;
 import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
@@ -51,6 +62,8 @@ import com.sencha.gxt.widget.core.client.container.MarginData;
 import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
 import com.sencha.gxt.widget.core.client.event.BeforeShowContextMenuEvent;
 import com.sencha.gxt.widget.core.client.event.BeforeShowContextMenuEvent.BeforeShowContextMenuHandler;
+import com.sencha.gxt.widget.core.client.event.SelectEvent;
+import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;
 import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
 import com.sencha.gxt.widget.core.client.grid.ColumnModel;
 import com.sencha.gxt.widget.core.client.grid.Grid;
@@ -92,8 +105,11 @@ public class ResourcesPanel extends FramedPanel {
 	private boolean drawed = false;
 	private Menu contextMenu;
 	private MenuItem openItem;
+	private MenuItem saveItem;
 	private MenuItem removeItem;
-	private RemoveResourceSession removeResourceSession;
+
+	private RemoveResourceSession removeResourceSession;	
+	private SaveResourceSession saveResourceSession;
 
 	public ResourcesPanel(ResourcesDialog parent, TRId trId, EventBus eventBus) {
 		super();
@@ -166,82 +182,92 @@ public class ResourcesPanel extends FramedPanel {
 					@Override
 					public void render(Context context,
 							ResourceTDDescriptor value, SafeHtmlBuilder sb) {
-						String data=
-								"<table style='font-family: tahoma, arial, verdana, sans-serif;"
-										+ "width: 100%; border: none;"
-										+ "font-size: 12px; margin:2px; text-align: left;'>"
-										+ "<tr>"
-										+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Name: </b></td>"
-										+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"+SafeHtmlUtils.htmlEscape(value.getName())+
-						
-										"</td>"
-												+ "</tr>"
-												+ "<tr>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Description: </b></td>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"+SafeHtmlUtils.htmlEscape(value.getDescription())+
-								
-										"</td>"
-												+ "</tr>"
-												+ "<tr>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Creation Date: </b></td>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"+
-												SafeHtmlUtils.htmlEscape(value.getCreationDate())+
-								
-										"</td>"
-												+ "</tr>"
-												+ "<tr>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Creator Id: </b></td>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"+new SafeHtmlBuilder().append(value.getCreatorId()).toSafeHtml().asString()+
-								
-										"</td>"
-												+ "</tr>"
-												+ "<tr>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Type: </b></td>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+						String data = "<table style='font-family: tahoma, arial, verdana, sans-serif;"
+								+ "width: 100%; border: none;"
+								+ "font-size: 12px; margin:2px; text-align: left;'>"
+								+ "<tr>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Name: </b></td>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+								+ SafeHtmlUtils.htmlEscape(value.getName())
 								+
-								SafeHtmlUtils.htmlEscape(value.getResourceType().toString())+"</td>" + "</tr>";
+
+								"</td>"
+								+ "</tr>"
+								+ "<tr>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Description: </b></td>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+								+ SafeHtmlUtils.htmlEscape(value
+										.getDescription())
+								+
+
+								"</td>"
+								+ "</tr>"
+								+ "<tr>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Creation Date: </b></td>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+								+ SafeHtmlUtils.htmlEscape(value
+										.getCreationDate())
+								+
+
+								"</td>"
+								+ "</tr>"
+								+ "<tr>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Creator Id: </b></td>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+								+ new SafeHtmlBuilder()
+										.append(value.getCreatorId())
+										.toSafeHtml().asString()
+								+
+
+								"</td>"
+								+ "</tr>"
+								+ "<tr>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Type: </b></td>"
+								+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+								+ SafeHtmlUtils.htmlEscape(value
+										.getResourceType().toString())
+								+ "</td>" + "</tr>";
 
 						ResourceTD resource = value.getResourceTD();
-						String resourceData="";
+						String resourceData = "";
 						if (resource instanceof InternalURITD) {
 							InternalURITD internalURITD = (InternalURITD) resource;
-							resourceData=
-									"<tr>"
-											+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>File Id: </b></td>"
-											+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"+SafeHtmlUtils.htmlEscape(internalURITD.getFileId())+
-									"</td>" + "</tr>";
+							resourceData = "<tr>"
+									+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>File Id: </b></td>"
+									+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+									+ SafeHtmlUtils.htmlEscape(internalURITD
+											.getFileId()) + "</td>" + "</tr>";
 
-							
 						} else {
 							if (resource instanceof StringResourceTD) {
 								StringResourceTD stringResourceTD = (StringResourceTD) resource;
-								resourceData=
-										"<tr>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Value: </b></td>"
-												+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"+
-												SafeHtmlUtils.htmlEscape(
-												stringResourceTD
-														.getStringValue())+
-										"</td>" + "</tr>";
+								resourceData = "<tr>"
+										+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Value: </b></td>"
+										+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+										+ SafeHtmlUtils
+												.htmlEscape(stringResourceTD
+														.getStringValue())
+										+ "</td>" + "</tr>";
 
 							} else {
 								if (resource instanceof TableResourceTD) {
 									TableResourceTD tableResourceTD = (TableResourceTD) resource;
-									resourceData=
-											"<tr>"
-													+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Table Id: </b></td>"
-													+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"+
-													new SafeHtmlBuilder().append(tableResourceTD
-													.getTableId()).toSafeHtml().asString()+
-											
-													"</td>" + "</tr>";
+									resourceData = "<tr>"
+											+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'><b>Table Id: </b></td>"
+											+ "<td style='font-size: 12px; padding: 2px 1px 1px 1px;'>"
+											+ new SafeHtmlBuilder()
+													.append(tableResourceTD
+															.getTableId())
+													.toSafeHtml().asString() +
+
+											"</td>" + "</tr>";
 
 								} else {
 
 								}
 							}
 						}
-						sb.appendHtmlConstant(data+resourceData+"</table>");
+						sb.appendHtmlConstant(data + resourceData + "</table>");
 
 					}
 
@@ -254,7 +280,9 @@ public class ResourcesPanel extends FramedPanel {
 
 			@Override
 			public void render(Context context, String value, SafeHtmlBuilder sb) {
-				sb.appendHtmlConstant("<span title='"+SafeHtmlUtils.htmlEscape(value)+"'>"+SafeHtmlUtils.htmlEscape(value)+"</span>");
+				sb.appendHtmlConstant("<span title='"
+						+ SafeHtmlUtils.htmlEscape(value) + "'>"
+						+ SafeHtmlUtils.htmlEscape(value) + "</span>");
 
 			}
 
@@ -262,53 +290,35 @@ public class ResourcesPanel extends FramedPanel {
 
 		ColumnConfig<ResourceTDDescriptor, ResourceTDType> typeColumn = new ColumnConfig<ResourceTDDescriptor, ResourceTDType>(
 				props.resourceType(), 30, "Type");
+	
+		ResourceTDTypeButtonCell button = new ResourceTDTypeButtonCell();
+		button.addSelectHandler(new SelectHandler() {
 
-		typeColumn.setCell(new AbstractCell<ResourceTDType>() {
 			@Override
-			public void render(Context context, ResourceTDType value,
-					SafeHtmlBuilder sb) {
-				String head=("<img title='"+SafeHtmlUtils.htmlEscape(value.toString())+"' src='");
-				String bodyC;
-				switch (value) {
-				case CHART:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.chart()
-							.getSafeUri().asString());
-					break;
-				case CODELIST:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.codelist()
-							.getSafeUri().asString());
-					break;
-				case CSV:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.csv().getSafeUri()
-							.asString());
-					break;
-				case GUESSER:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.resources()
-							.getSafeUri().asString());
-					break;
-				case JSON:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.json()
-							.getSafeUri().asString());
-					break;
-				case MAP:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.gis().getSafeUri()
-							.asString());
-					break;
-				case SDMX:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.sdmx()
-							.getSafeUri().asString());
-					break;
-				default:
-					bodyC=SafeHtmlUtils.htmlEscape(ResourceBundle.INSTANCE.resources()
-							.getSafeUri().asString());
-					break;
-
-				}
-				sb.appendHtmlConstant(head+bodyC+"' >");
-
+			public void onSelect(SelectEvent event) {
+				Log.debug("Button  Pressed");
+				
+				Context c = event.getContext();
+				int rowIndex = c.getIndex();
+				int columnIndex=c.getColumn();
+				
+				Element el=grid.getView().getCell(rowIndex, columnIndex);
+				
+				NativeEvent contextEvent = Document.get().createMouseEvent(BrowserEvents.CONTEXTMENU, true, true,0, 0, 0,
+				        el.getAbsoluteLeft(), el.getAbsoluteTop(), false, false, false, false,
+				        NativeEvent.BUTTON_RIGHT, null);
+				
+				//NativeEvent contextEvent = Document.get().createContextMenuEvent();		
+				el.dispatchEvent(contextEvent);
+				//DomEvent
 			}
 		});
 
+		typeColumn.setCell(button);
+
+	
+		
+		
 		List<ColumnConfig<ResourceTDDescriptor, ?>> l = new ArrayList<ColumnConfig<ResourceTDDescriptor, ?>>();
 		l.add(expander);
 		l.add(nameCol);
@@ -348,6 +358,9 @@ public class ResourcesPanel extends FramedPanel {
 					}
 				});
 			}
+			
+			
+			
 		};
 
 		grid.setLoader(loader);
@@ -390,6 +403,21 @@ public class ResourcesPanel extends FramedPanel {
 
 		});
 
+		saveItem = new MenuItem();
+		saveItem.setText("Save");
+		saveItem.setIcon(ResourceBundle.INSTANCE.save());
+		saveItem.addSelectionHandler(new SelectionHandler<Item>() {
+
+			@Override
+			public void onSelection(SelectionEvent<Item> event) {
+				ResourceTDDescriptor selected = grid.getSelectionModel()
+						.getSelectedItem();
+				Log.debug("selected: " + selected);
+				requestSave(selected);
+			}
+
+		});
+
 		removeItem = new MenuItem();
 		removeItem.setText("Delete");
 		removeItem.setIcon(ResourceBundle.INSTANCE.delete());
@@ -419,6 +447,9 @@ public class ResourcesPanel extends FramedPanel {
 				switch (resourceTDType) {
 				case CHART:
 					contextMenu.clear();
+					openItem.setIcon(ResourceBundle.INSTANCE.chart());
+					contextMenu.add(openItem);
+					contextMenu.add(saveItem);
 					contextMenu.add(removeItem);
 					grid.setContextMenu(contextMenu);
 					break;
@@ -429,6 +460,7 @@ public class ResourcesPanel extends FramedPanel {
 					break;
 				case CSV:
 					contextMenu.clear();
+					contextMenu.add(saveItem);
 					contextMenu.add(removeItem);
 					grid.setContextMenu(contextMenu);
 					break;
@@ -439,6 +471,7 @@ public class ResourcesPanel extends FramedPanel {
 					break;
 				case JSON:
 					contextMenu.clear();
+					contextMenu.add(saveItem);
 					contextMenu.add(removeItem);
 					grid.setContextMenu(contextMenu);
 					break;
@@ -468,6 +501,97 @@ public class ResourcesPanel extends FramedPanel {
 
 	}
 
+	protected void requestSave(ResourceTDDescriptor resourceTDDescriptor) {
+		switch (resourceTDDescriptor.getResourceType()) {
+		case CHART:
+			requestSaveResource(resourceTDDescriptor, ".jpg", "image/jpeg");
+			break;
+		case CODELIST:
+			break;
+		case CSV:
+			requestSaveResource(resourceTDDescriptor, ".csv", "text/csv");
+			break;
+		case GUESSER:
+			break;
+		case JSON:
+			requestSaveResource(resourceTDDescriptor, ".json",
+					"application/json");
+			break;
+		case MAP:
+			break;
+		case SDMX:
+			break;
+		default:
+			break;
+
+		}
+
+	}
+
+	protected void requestSaveResource(
+			ResourceTDDescriptor resourceTDDescriptor, String extension,
+			String mimeType) {
+		
+		InternalURITD internalURITD;
+		ResourceTD resource = resourceTDDescriptor.getResourceTD();
+		if (resource instanceof InternalURITD) {
+			internalURITD=(InternalURITD)resource;
+		} else {
+			UtilsGXT3.alert("Attention", "This resource does not have valid internal URI!");
+			Log.debug("Attention, this resource does not have valid Internal URI!");
+			return;
+		}
+		Log.debug("InteranlURI: "+internalURITD);
+		
+		saveResourceSession = new SaveResourceSession();
+		saveResourceSession.setMimeType(mimeType);
+		saveResourceSession.setExtension(extension);
+		saveResourceSession.setFileName(resourceTDDescriptor.getName()
+				+ extension);
+		saveResourceSession.setFileDescription(resourceTDDescriptor
+				.getDescription());
+		saveResourceSession.setUri(internalURITD.getUri());
+		
+		GWT.runAsync(new RunAsyncCallback() {
+
+			public void onSuccess() {
+
+				SaveResourceWizard saveResourceWizard = new SaveResourceWizard(
+						saveResourceSession, "Save Resource", eventBus);
+
+				saveResourceWizard.addListener(new WizardListener() {
+
+					public void failed(String title, String message, String details,
+							Throwable throwable) {
+						Log.debug(title + ", " + message+" "+details);
+					}
+
+					public void completed(TRId id) {
+						Log.debug("Save Resource Completed");
+
+					}
+
+					@Override
+					public void putInBackground() {
+						Log.debug("PutInBakground");
+					}
+
+					public void aborted() {
+						Log.debug("Save Resource Aborted");
+					}
+				});
+
+				saveResourceWizard.show();
+			}
+
+			public void onFailure(Throwable reason) {
+				Log.error("Async code loading failed", reason);
+				eventBus.fireEvent(new SessionExpiredEvent(
+						SessionExpiredType.EXPIREDONSERVER));
+			}
+		});
+	}
+
 	protected void requestRemove(ResourceTDDescriptor resourceTDDescriptor) {
 
 		removeResourceSession = new RemoveResourceSession(trId,
@@ -489,7 +613,8 @@ public class ResourcesPanel extends FramedPanel {
 								Log.error("Error removing the resource: "
 										+ caught.getLocalizedMessage());
 								UtilsGXT3.alert("Error",
-										"Error removing the resource: "+caught.getLocalizedMessage());
+										"Error removing the resource: "
+												+ caught.getLocalizedMessage());
 							}
 						}
 					}
@@ -497,7 +622,7 @@ public class ResourcesPanel extends FramedPanel {
 					public void onSuccess(Void v) {
 						Log.debug("Resource removed");
 						grid.getLoader().load();
-						//UtilsGXT3.info("Resource", "Resource Removed!");
+						// UtilsGXT3.info("Resource", "Resource Removed!");
 					}
 
 				});
@@ -507,6 +632,7 @@ public class ResourcesPanel extends FramedPanel {
 	protected void requestOpen(ResourceTDDescriptor resourceTDDescriptor) {
 		switch (resourceTDDescriptor.getResourceType()) {
 		case CHART:
+			repquestOpenChart(resourceTDDescriptor);
 			break;
 		case CODELIST:
 			break;
@@ -528,6 +654,12 @@ public class ResourcesPanel extends FramedPanel {
 
 	}
 
+	private void repquestOpenChart(ResourceTDDescriptor resourceTDDescriptor) {
+		ChartViewerDialog chartDialog=new ChartViewerDialog(resourceTDDescriptor, trId, eventBus);
+		chartDialog.show();
+		
+	}
+
 	protected void requestOpenMap(
 			final ResourceTDDescriptor resourceTDDescriptor) {
 		ResourceTD resource = resourceTDDescriptor.getResourceTD();
@@ -613,11 +745,11 @@ public class ResourcesPanel extends FramedPanel {
 
 					public void onSuccess(ArrayList<ResourceTDDescriptor> result) {
 						Log.debug("loaded " + result.size());
-						try{
+						try {
 							callback.onSuccess(new ListLoadResultBean<ResourceTDDescriptor>(
-								result));
-						} catch(Throwable e){
-							Log.debug("Error: "+e.getLocalizedMessage());
+									result));
+						} catch (Throwable e) {
+							Log.debug("Error: " + e.getLocalizedMessage());
 							e.printStackTrace();
 						}
 					}
